@@ -33,12 +33,17 @@
 
 static snd_pcm_t *handle = NULL;
 static snd_pcm_uframes_t written_frames = 0;
+static long drain_delta = 0 ;
+static long last_processed_bytes0 = -1 ;
+static long last_processed_bytes = -1 ;
+snd_pcm_uframes_t buffer_total_frames; /* number of frames in alsa device buffer */
 
 static void snd_perr(char *text, int err)
 {
     fprintf(stderr, "##########################################################\n");
     fprintf(stderr, "%s\n", text);
     fprintf(stderr, "%s\n", snd_strerror(err));
+    warning(text) ;
 }
 
 int audio_device_open(char *output_device)
@@ -46,16 +51,20 @@ int audio_device_open(char *output_device)
     int err = snd_pcm_open(&handle, output_device, /*"default",*/
                            SND_PCM_STREAM_PLAYBACK, SND_PCM_NONBLOCK);
     if (err < 0) {
-        snd_perr("audio_device_open: snd_pcm_open", err);
+        snd_perr("ALSA audio_device_open: snd_pcm_open", err);
         return -1;
     }
 
     written_frames = 0;
+    drain_delta=0 ;
+    last_processed_bytes0 = -1 ;
+    last_processed_bytes = -1 ;
     return 0;
 }
 
 int audio_device_set_params(AUDIO_FORMAT *format, int *channels, int *rate)
 {
+    unsigned int utmp ;
     int err;
     snd_pcm_format_t alsa_format;
     snd_pcm_hw_params_t *params;
@@ -66,15 +75,15 @@ int audio_device_set_params(AUDIO_FORMAT *format, int *channels, int *rate)
 
     err = snd_pcm_hw_params_any(handle, params);
     if (err < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_any", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_any", err);
         return -1;
     }
 
-    err = snd_pcm_hw_params_set_access(handle, params, SND_PCM_ACCESS_RW_INTERLEAVED);
-    if (err < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_set_access", err);
-        return -1;
-    }
+/*      err = snd_pcm_hw_params_set_access(handle, params, SND_PCM_ACCESS_RW_INTERLEAVED);  */
+/*      if (err < 0) {  */
+/*          snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_set_access", err);  */
+/*          return -1;  */
+/*      }  */
 
 
 
@@ -88,11 +97,11 @@ int audio_device_set_params(AUDIO_FORMAT *format, int *channels, int *rate)
     }
 
     if (snd_pcm_hw_params_set_format(handle, params, alsa_format) < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_set_format", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_set_format", err);
         return -1;
     }
     if (snd_pcm_hw_params_get_format(params, &alsa_format) < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_get_format", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_get_format", err);
         return -1;
     }
 
@@ -109,35 +118,41 @@ int audio_device_set_params(AUDIO_FORMAT *format, int *channels, int *rate)
 
     err = snd_pcm_hw_params_set_channels(handle, params, *channels);
     if (err < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_set_channels", err);
-        return -1;
-    }
-    if (snd_pcm_hw_params_get_channels(params, channels) < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_get_channels", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_set_channels", err);
         return -1;
     }
 
+    utmp = (unsigned int)*channels ;
+
+    if (snd_pcm_hw_params_get_channels(params, &utmp) < 0) {
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_get_channels", err);
+        return -1;
+    }
+    *channels = (int)utmp ;
 
 
-    err = snd_pcm_hw_params_set_rate_near(handle, params, rate, 0);
+
+    utmp = (unsigned int)*rate ;
+    err = snd_pcm_hw_params_set_rate_near(handle, params, &utmp, 0);
     if (err < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params_set_rate_near", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params_set_rate_near", err);
         return -1;
     }
+    *rate = (int)utmp ;
 
 
     err = snd_pcm_hw_params(handle, params);
     if (err < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_hw_params", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_hw_params", err);
         return -1;
     }
 
     err = snd_pcm_prepare(handle);
     if (err < 0) {
-        snd_perr("audio_device_set_params: snd_pcm_prepare", err);
+        snd_perr("ALSA audio_device_set_params: snd_pcm_prepare", err);
         return -1;
     }
-      
+
     return 0;
 }
 
@@ -153,7 +168,7 @@ static int recover_snd_handle(int err)
     if (err == -EPIPE) { /* underrun */
         err = snd_pcm_prepare(handle);
         if (err < 0)
-            snd_perr("recover_snd_handle: can't recover underrun, prepare failed", err);
+            snd_perr("ALSA recover_snd_handle: can't recover underrun, prepare failed", err);
         return 0;
     }
     else if (err == -ESTRPIPE) { /* suspend */
@@ -163,7 +178,7 @@ static int recover_snd_handle(int err)
         if (err < 0) {
             err = snd_pcm_prepare(handle);
             if (err < 0)
-                snd_perr("recover_snd_handle: can't recover suspend, prepare failed", err);
+                snd_perr("ALSA recover_snd_handle: can't recover suspend, prepare failed", err);
         }
         return 0;
     }
@@ -183,7 +198,7 @@ int audio_device_write(unsigned char *data, int count)
         }
         else if (err < 0) {
             if (recover_snd_handle(err) < 0) {
-                snd_perr("audio_device_write: snd_pcm_writei", err);
+                snd_perr("ALSA audio_device_write: snd_pcm_writei", err);
                 return -1;
             }
         }
@@ -200,56 +215,59 @@ int audio_device_write(unsigned char *data, int count)
     return snd_pcm_frames_to_bytes(handle, result_frames);
 }
 
-void audio_device_close(void)
+/* Number of bytes processed since opening the device. */
+long query_processed_bytes(void)
+{
+    if(handle != NULL) {
+	snd_pcm_sframes_t avail_frames_in_buf = snd_pcm_avail_update(handle);
+	return snd_pcm_frames_to_bytes(handle, (written_frames - (buffer_total_frames - avail_frames_in_buf)));
+    }
+
+    return 0 ;
+}
+
+
+long _audio_device_processed_bytes = 0 ;
+
+/* Number of bytes processed since opening the device. */
+long audio_device_processed_bytes(void)
+{
+    if(handle != NULL)
+	_audio_device_processed_bytes = query_processed_bytes() ;
+
+    return _audio_device_processed_bytes ;
+}
+
+void audio_device_close(int drain)
 {
     if (handle != NULL) {
         int err;
 
 	printf("Closing the ALSA audio device\n") ;
 
-        err = snd_pcm_drain(handle);
+	_audio_device_processed_bytes = query_processed_bytes() ;
+
+	if(drain)
+	    err = snd_pcm_drain(handle);
 
         err = snd_pcm_drop(handle);
         if (err < 0) {
-            snd_perr("audio_device_close: snd_pcm_drop", err);
+            snd_perr("ALSA audio_device_close: snd_pcm_drop", err);
         }
 
         err = snd_pcm_close(handle);
         if (err < 0) {
-            snd_perr("audio_device_close: snd_pcm_close", err);
+            snd_perr("ALSA audio_device_close: snd_pcm_close", err);
         }
 
         handle = NULL;
     }
-}
-
-/* Number of bytes processed since opening the device. */
-long audio_device_processed_bytes(void)
-{
-    snd_pcm_sframes_t frame_delay = 0;
-
-    int err = snd_pcm_delay(handle, &frame_delay);
-
-    if (err < 0) {
-	if (err == -EPIPE)
-	    snd_perr("audio_device_processed_bytes: ALSA buffer underrun", err);
-	else
-	    snd_perr("audio_device_processed_bytes: snd_pcm_delay", err);
-
-	if (recover_snd_handle(err) < 0) {
-	    fprintf(stderr, "audio_device_processed_bytes: could not recover handle\n");
-	}
-
-	return snd_pcm_frames_to_bytes(handle, written_frames);
-    }
-
-    return snd_pcm_frames_to_bytes(handle, written_frames - frame_delay);
+    drain_delta=0 ;
 }
 
 int audio_device_best_buffer_size(int playback_bytes_per_block)
 {
     int err;
-    snd_pcm_uframes_t frames;
     snd_pcm_status_t *status;
     int frame_size ;
 
@@ -257,15 +275,15 @@ int audio_device_best_buffer_size(int playback_bytes_per_block)
 
     err = snd_pcm_status(handle, status);
     if (err < 0) {
-        snd_perr("audio_device_best_buffer_size: snd_pcm_status", err);
+        snd_perr("ALSA audio_device_best_buffer_size: snd_pcm_status", err);
         return 0;
     }
 
-    frames = snd_pcm_status_get_avail(status);
+    buffer_total_frames = snd_pcm_status_get_avail(status);
 
-    frame_size = snd_pcm_frames_to_bytes(handle, frames);
+    frame_size = snd_pcm_frames_to_bytes(handle, buffer_total_frames);
 
-    printf("ALSA audio_device_best_buffer_size:%d (frames:%ld)\n", frame_size, frames) ;
+/*      printf("ALSA audio_device_best_buffer_size:%d (frames:%ld)\n", frame_size, buffer_total_frames) ;  */
 
 
     if(frame_size < 4096 && frame_size > 0) {
