@@ -24,11 +24,23 @@
 #include <sys/stat.h>
 #include <sys/mman.h>
 #include <memory.h>
+#include <endian.h>
 #include <fcntl.h>
 #include <unistd.h>
 #include <gtk/gtk.h>
 #include <sndfile.h>
+
+#ifdef HAVE_OGG
+#include "vorbis/codec.h"
+#include "vorbis/vorbisfile.h"
+#endif
+
 #include "gwc.h"
+
+#ifdef HAVE_MP3
+#include "mpg123.h"
+#endif
+
 #include "fmtheaders.h"
 #include "encoding.h"
 #include "audio_device.h"
@@ -44,12 +56,34 @@ int BYTESPERSAMPLE = 2 ;
 int MAXSAMPLEVALUE = 1 ;
 int PLAYBACK_FRAMESIZE = 4 ;
 int FRAMESIZE = 4 ;
+int current_ogg_bitstream = 0 ;
+int nonzero_seek ;
 /*  int dump_sample = 0 ;  */
 long wavefile_data_start ;
 
 SNDFILE      *sndfile = NULL ;
 SF_INFO      sfinfo ;
 
+#ifdef HAVE_OGG
+OggVorbis_File oggfile ;
+#endif
+
+FILE *fp_ogg = NULL ;
+
+#ifdef HAVE_MP3
+mpg123_handle *fp_mp3 = NULL ;
+#endif
+
+int audiofileisopen = 0 ;
+long current_ogg_or_mp3_pos ;
+
+#define SNDFILE_TYPE 0x01
+#define OGG_TYPE 0x02
+#define MP3_TYPE 0x04
+
+int audio_type ;
+
+
 extern struct view audio_view ;
 extern struct sound_prefs prefs ;
 extern struct encoding_prefs encoding_prefs;
@@ -60,10 +94,12 @@ void position_wavefile_pointer(long sample_number) ;
 
 void audio_normalize(int flag)
 {
-    if(flag == 0)
-	sf_command(sndfile, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
-    else
-	sf_command(sndfile, SFC_SET_NORM_DOUBLE, NULL, SF_TRUE) ;
+    if(audio_type == SNDFILE_TYPE) {
+	if(flag == 0)
+	    sf_command(sndfile, SFC_SET_NORM_DOUBLE, NULL, SF_FALSE) ;
+	else
+	    sf_command(sndfile, SFC_SET_NORM_DOUBLE, NULL, SF_TRUE) ;
+    }
 }
 
 void write_wav_header(int thefd, int speed, long bcount, int bits, int stereo)
@@ -105,7 +141,13 @@ void config_audio_device(int rate_set, int bits_set, int stereo_set)
 
     bits_set = 16 ;
     /* play everything as 16 bit, signed integers */
+    /* using the appropriate endiannes */
+
+#if __BYTE_ORDER == __BIG_ENDIAN
+    format_set = GWC_S16_BE ;
+#else
     format_set = GWC_S16_LE ;
+#endif
 
     rate = rate_set ;
     audio_bits = bits_set ;
@@ -125,7 +167,7 @@ void config_audio_device(int rate_set, int bits_set, int stereo_set)
 
     if(format != format_set) {
 	char *buf_fmt_str ;
-	char buf[80] ;
+	char buf[85] ;
 	switch(format_set) {
 	    case GWC_U8 : buf_fmt_str = "8 bit (unsigned)" ; bits_set = 8 ; break ;
 	    case GWC_S8 : buf_fmt_str = "8 bit (signed)" ; bits_set = 8 ; break ;
@@ -133,23 +175,23 @@ void config_audio_device(int rate_set, int bits_set, int stereo_set)
 	    case GWC_S16_LE : buf_fmt_str = "16 bit" ; bits_set = 16 ; break ;
 	    default : buf_fmt_str = "unknown!" ; bits_set = 8 ; break ;
 	}
-        sprintf(buf, "Set bits to %s - does your soundcard support what you requested?\n", buf_fmt_str) ;
+        snprintf(buf, sizeof(buf), "Set bits to %s - does your soundcard support what you requested?\n", buf_fmt_str) ;
 	warning(buf) ;
     }
 
     if(channels != stereo + 1) {
 	char buf[80] ;
 	if(stereo == 0)
-	    sprintf(buf, "Failed to set mono mode\nYour sound card may not support mono\n") ;
+	    snprintf(buf, sizeof(buf), "Failed to set mono mode\nYour sound card may not support mono\n") ;
 	else
-	    sprintf(buf, "Failed to set stereo mode\nYour sound card may not support stereo\n") ;
+	    snprintf(buf, sizeof(buf), "Failed to set stereo mode\nYour sound card may not support stereo\n") ;
 	warning(buf) ;
     }
     stereo_set = channels - 1 ;
 
     if(ABS(rate_set-rate) > 10) {
 	char buf[80] ;
-	sprintf(buf, "Rate set to %d instead of %d\nYour sound card may not support the desired rate\n",
+	snprintf(buf, sizeof(buf), "Rate set to %d instead of %d\nYour sound card may not support the desired rate\n",
 	             rate_set, rate) ;
 	warning(buf) ;
     }
@@ -167,6 +209,7 @@ int looped_count ;
 #define MAXBUFSIZE 32768
 int BUFSIZE ;
 unsigned char audio_buffer[MAXBUFSIZE] ;
+unsigned char audio_buffer2[MAXBUFSIZE] ;
 
 long playback_start_position ;
 long playback_end_position ;
@@ -201,15 +244,22 @@ long start_playback(char *output_device, struct view *v, struct sound_prefs *p,
     long playback_samples ;
     gfloat lv, rv ;
 
-    if(sndfile == NULL) return 1 ;
+    if(audio_type == SNDFILE_TYPE && sndfile == NULL) return 1 ;
+#ifdef HAVE_OGG
+    if(audio_type == OGG_TYPE && fp_ogg == NULL) return 1 ;
+#endif
 
-    audio_device_close() ;
+    audio_device_close(1) ;
 
     if (audio_device_open(output_device) == -1) {
-	char buf[80] ;
-	sprintf(buf, "Failed to open output device %s", output_device) ;
+	char buf[255] ;
+#ifdef HAVE_ALSA
+	snprintf(buf, sizeof(buf), "Failed to open alsa output device %s, check settings->miscellaneous for device information", output_device) ;
+#else
+	snprintf(buf, sizeof(buf), "Failed to open output device %s, check settings->miscellaneous for device information", output_device) ;
+#endif
 	warning(buf) ;
-	return 1 ;
+	return 0 ;
     }
 
     get_region_of_interest(&first, &last, v) ;
@@ -249,7 +299,8 @@ long start_playback(char *output_device, struct view *v, struct sound_prefs *p,
 
     audio_state = AUDIO_IS_PLAYBACK ;
 
-    sf_seek(sndfile, playback_position, SEEK_SET) ;
+    position_wavefile_pointer(playback_start_position) ;
+/*      g_print("playback_start_position is %ld\n", playback_start_position) ;  */
 
     /* put some data in the buffer queues, to avoid underflows */
     if(0) {
@@ -263,7 +314,6 @@ long start_playback(char *output_device, struct view *v, struct sound_prefs *p,
 	playback_bytes_per_block = old_playback_bytes ;
     }
 
-/*      g_print("playback_start_position is %ld\n", playback_start_position) ;  */
 /*      g_print("playback_samples is %ld\n", playback_samples) ;  */
 /*      g_print("BUFSIZE %ld (%lg fragments)\n", (long)BUFSIZE, (double)BUFSIZE/(double)oss_info.fragsize) ;  */
 
@@ -333,27 +383,50 @@ void truncate_wavfile(struct view *v)
 }
 #endif /* !TRUNCATE_OLD */
 
+void sndfile_truncate(long n_samples)
+{
+    sf_count_t total_samples =  n_samples ;
+    if(sf_command(sndfile, SFC_FILE_TRUNCATE, &total_samples, sizeof(total_samples)))
+	warning("Libsndfile reports truncation of audio file failed") ;
+}
+
 int close_wavefile(struct view *v)
 {
+    if(audio_type == SNDFILE_TYPE) {
 #ifdef TRUNCATE_OLD
-    int r ;
+	int r ;
 
-    if(v->truncate_head > 0 || v->truncate_tail < v->n_samples -1) {
-	r = yesnocancel("Part of the waveform is selected for truncation, do you really want to truncate?") ;
-	if(r == 2) return 0 ;
-	if(r == 0) truncate_wavfile(v) ;
-    }
+	if(v->truncate_head > 0 || v->truncate_tail < v->n_samples -1) {
+	    r = yesnocancel("Part of the waveform is selected for truncation, do you really want to truncate?") ;
+	    if(r == 2) return 0 ;
+	    if(r == 0) truncate_wavfile(v) ;
+	}
 #endif /* TRUNCATE_OLD */
-    if(sndfile != NULL) {
-	sf_close(sndfile) ;
+	if(sndfile != NULL) {
+	    sf_close(sndfile) ;
+	}
+	audio_device_close(0) ;
+	sndfile = NULL ;
+#ifdef HAVE_OGG
+    } else if(audio_type == OGG_TYPE) {
+	if(fp_ogg != NULL) {
+	    ov_clear(&oggfile) ;
+	}
+	fp_ogg = NULL ;
+#endif
+#ifdef HAVE_MP3
+    } else if(audio_type == MP3_TYPE) {
+	if(fp_mp3 != NULL) {
+	    mpg123_close(fp_mp3) ;
+	}
+	fp_mp3 = NULL ;
+#endif
     }
-    audio_device_close() ;
-    sndfile = NULL ;
 
     return 1 ;
 }
 
-void save_selection_as_wavfile(char *filename_new, struct view *v)
+void save_as_wavfile(char *filename_new, long first_sample, long last_sample)
 {
     int fd_new ;
     SNDFILE *sndfile_new ;
@@ -362,51 +435,37 @@ void save_selection_as_wavfile(char *filename_new, struct view *v)
     long total_samples ;
     long total_bytes ;
 
-    total_samples =  v->selected_last_sample-v->selected_first_sample+1 ;
+    total_samples =  last_sample-first_sample+1 ;
 
-    if(total_samples < 0 || total_samples > v->n_samples) {
+    if(total_samples < 0) {
 	warning("Invalid selection") ;
 	return ;
     }
 
     total_bytes = total_samples*FRAMESIZE ;
 
-    {
-	char buf[1000] ;
-	sprintf(buf, "Save %ld bytes to %s", total_bytes, filename_new) ;
-	if(yesno(buf))  {
-	    return ;
-	}
-    }
-
     fd_new = open(filename_new, O_RDONLY) ;
 
     if(fd_new > -1) {
-	char buf[1000] ;
+	char buf[PATH_MAX] ;
 	close(fd_new) ;
-	sprintf(buf, "%s exists, overwrite ?", filename_new) ;
+	snprintf(buf, sizeof(buf), "%s exists, overwrite ?", filename_new) ;
 	if(yesno(buf))  {
 	    return ;
 	}
     }
 
 
-/*      fd_new = open(filename_new, O_WRONLY|O_CREAT|O_TRUNC,S_IRGRP|S_IROTH|S_IRUSR|S_IWUSR) ;  */
-
     sfinfo_new = sfinfo ;
     sfinfo_new.frames = total_samples ;
 
     if (! (sndfile_new = sf_open (filename_new, SFM_WRITE, &sfinfo_new))) {
 	/* Open failed so print an error message. */
 
-	char buf[80] ;
-	sprintf(buf, "Failed to save selection %s", filename_new) ;
+	char buf[PATH_MAX] ;
+	snprintf(buf, sizeof(buf), "Failed to save selection %s", filename_new) ;
 	warning(buf) ;
 	return ;
-
-        /* Print the error message from libsndfile. */
-/*          sf_perror (NULL) ;  */
-/*          return  1 ;  */
     } ;
 
     push_status_text("Saving selection") ;
@@ -415,7 +474,7 @@ void save_selection_as_wavfile(char *filename_new, struct view *v)
     /* something like this, gotta buffer this or the disk head will
        burn a hole in the platter */
 
-    position_wavefile_pointer(v->selected_first_sample) ;
+    position_wavefile_pointer(first_sample) ;
 
     {
 	long n_copied  ;
@@ -451,6 +510,53 @@ void save_selection_as_wavfile(char *filename_new, struct view *v)
 
 }
 
+void save_selection_as_wavfile(char *filename_new, struct view *v)
+{
+    int fd_new ;
+    SNDFILE *sndfile_new ;
+    SF_INFO sfinfo_new ;
+
+    long total_samples ;
+    long total_bytes ;
+
+    total_samples =  v->selected_last_sample-v->selected_first_sample+1 ;
+
+    if(total_samples < 0 || total_samples > v->n_samples) {
+	warning("Invalid selection") ;
+	return ;
+    }
+
+    save_as_wavfile(filename_new, v->selected_first_sample, v->selected_last_sample) ;
+}
+
+#ifdef HAVE_MP3
+int gwc_mpg123_open(char *filename)
+{
+    int r ;
+
+    mpg123_init() ;
+
+    fp_mp3 = mpg123_new(NULL,NULL) ;
+
+    r = mpg123_open(fp_mp3,filename) ;
+
+    if(r != MPG123_OK) {
+	mpg123_delete(fp_mp3) ;
+	fp_mp3 = NULL ;
+    }
+
+    return r ;
+}
+
+int gwc_mpg123_close(void)
+{
+    mpg123_close(fp_mp3) ;
+    mpg123_delete(fp_mp3) ;
+    mpg123_exit() ;
+    return 0 ;
+}
+#endif
+
 
 int is_valid_audio_file(char *filename)
 {
@@ -459,9 +565,40 @@ int is_valid_audio_file(char *filename)
 
     sfinfo.format = 0 ;
 
-    if((sndfile = sf_open(filename, SFM_READ, &sfinfo)) != NULL) {
+#ifdef HAVE_OGG
+    if((fp_ogg = fopen(filename, "r")) != NULL) {
+        if(ov_open(fp_ogg, &oggfile, NULL, 0) < 0) {
+            fclose(fp_ogg) ;
+            fp_ogg = NULL ;
+        } else {
+	    ov_clear(&oggfile) ;
+            fclose(fp_ogg) ;
+	    fp_ogg = NULL ;
+	    audio_type = OGG_TYPE ;
+	    return 1 ;
+	}
+    }
+#endif
+
+
+#ifdef HAVE_MP3
+    if(gwc_mpg123_open(filename) == MPG123_OK) {
+	gwc_mpg123_close() ;
+        fp_mp3 = NULL ;
+
+        audio_type = MP3_TYPE ;
+        return 1 ;
+    }
+#endif
+
+    if((sndfile = sf_open(filename, SFM_RDWR, &sfinfo)) != NULL) {
 	sf_close(sndfile) ;
+	audio_type = SNDFILE_TYPE ;
 	return 1 ;
+    } else {
+	char buf[180+PATH_MAX] ;
+	snprintf(buf, sizeof(buf), "Failed to open  %s, \'%s\'", filename, sf_strerror(NULL)) ;
+	warning(buf) ;
     }
 
     return 0 ;
@@ -472,6 +609,16 @@ struct sound_prefs open_wavefile(char *filename, struct view *v)
 {
     struct sound_prefs wfh ;
 
+    /* initialize all wfh structure members to defaults.  Will be overwritten on succesful file open */
+    wfh.rate = 44100 ;
+    wfh.n_channels = 2 ;
+    wfh.stereo = 1 ;
+    wfh.n_samples = 2 ;
+    wfh.playback_bits = wfh.bits = 16 ;
+    wfh.max_allowed = MAXSAMPLEVALUE-1 ;
+    wfh.wavefile_fd = wavefile_fd ;
+    wfh.sample_buffer_exists = FALSE ;
+
     if(close_wavefile(v)) {
 	wfh.successful_open = TRUE ;
     } else {
@@ -480,28 +627,63 @@ struct sound_prefs open_wavefile(char *filename, struct view *v)
     }
 
 
-    wfh.wavefile_fd = wavefile_fd ;
-    wfh.sample_buffer_exists = FALSE ;
+    if(audio_type == SNDFILE_TYPE) {
+	if (! (sndfile = sf_open (filename, SFM_RDWR, &sfinfo))) {
+	    /* Open failed so print an error message. */
 
-    if (! (sndfile = sf_open (filename, SFM_RDWR, &sfinfo))) {
-	/* Open failed so print an error message. */
+	    char buf[80+PATH_MAX] ;
+	    snprintf(buf, sizeof(buf), "Failed to open  %s, no permissions or unknown audio format", filename) ;
+	    warning(buf) ;
+	    wfh.successful_open = FALSE ;
+	    return wfh ;
 
-	char buf[80] ;
-	sprintf(buf, "Failed to open  %s, no permissions or unknown audio format", filename) ;
-	warning(buf) ;
-	wfh.successful_open = FALSE ;
-	return wfh ;
+	    /* Print the error message from libsndfile. */
+    /*          sf_perror (NULL) ;  */
+    /*          return  1 ;  */
+	} ;
+    }
+
+#ifdef HAVE_OGG
+    if(audio_type == OGG_TYPE) {
+        if((fp_ogg = fopen(filename, "r")) != NULL) {
+            if(ov_open(fp_ogg, &oggfile, NULL, 0) < 0) {
+                /* Open failed so print an error message. */
+
+                char buf[80+PATH_MAX] ;
+                snprintf(buf, sizeof(buf), "Failed to open  %s", filename) ;
+                warning(buf) ;
+		wfh.successful_open = FALSE ;
+		fclose(fp_ogg) ;
+		fp_ogg = NULL ;
+                return wfh ;
+            }
+        }
+
+    }
+#endif
+
+#ifdef HAVE_MP3
+    if(audio_type == MP3_TYPE) {
+	if(gwc_mpg123_open(filename) != MPG123_OK) {
+	    /* Open failed so print an error message. */
+
+	    char buf[80+PATH_MAX] ;
+	    snprintf(buf, sizeof(buf), "Failed to open  %s", filename) ;
+	    warning(buf) ;
+	    wfh.successful_open = FALSE ;
+	    fp_mp3 = NULL ;
+	    return wfh ;
+        }
+
+    }
+#endif
 
-        /* Print the error message from libsndfile. */
-/*          sf_perror (NULL) ;  */
-/*          return  1 ;  */
-    } ;
 
     wfh.wavefile_fd = 1 ;
 
 
-    /* determine soundfile properties */
-    {
+    if(audio_type == SNDFILE_TYPE) {
+	/* determine soundfile properties */
 	wfh.rate = sfinfo.samplerate ;
 	wfh.n_channels = sfinfo.channels ;
 	wfh.stereo = stereo = sfinfo.channels-1 ;
@@ -517,29 +699,69 @@ struct sound_prefs open_wavefile(char *filename, struct view *v)
 	    default : warning("Soundfile format not allowed") ; break ;
 	}
 
-	FRAMESIZE = BYTESPERSAMPLE*wfh.n_channels ;
-	PLAYBACK_FRAMESIZE = 2*wfh.n_channels ;
-
-	wfh.playback_bits = audio_bits = wfh.bits = BYTESPERSAMPLE*8 ;
-
-	wfh.max_allowed = MAXSAMPLEVALUE-1 ;
-
-#ifdef WAVHEADER_FIX
 	/* do some simple error checking on the wavfile header , so we don't seek data where it isn't */
-	if(!fstat(wavefile_fd, &wav_stat)) {
-	    if(wav_stat.st_size != wfh.data_offset+wfh.data_length) {
-		if(!yesno("Wave file header is damaged, attempt temporary fix?")) {
-		    header.data_length = wav_stat.st_size - wfh.data_offset ;
-
-		    wfh.n_samples = header.data_length / BYTESPERSAMPLE / wfh.n_channels ;
-		    wfh.data_length = header.data_length ;
-		    
-		}
+	if(wfh.n_samples < 2) {
+	    char tmp[140] ;
+	    snprintf(tmp, sizeof(tmp), "Audio file is possibly corrupt, only %ld samples reported by audio header", wfh.n_samples) ;
+	    info(tmp) ;
+	    if(sndfile != NULL) {
+		sf_close(sndfile) ;
 	    }
+	    wfh.successful_open = FALSE ;
+	    return wfh ;
 	}
-#endif
     }
 
+#ifdef HAVE_OGG
+    if(audio_type == OGG_TYPE) {
+        vorbis_info *vi = ov_info(&oggfile,-1) ;
+        wfh.rate = vi->rate ;
+        wfh.n_channels = vi->channels ;
+        wfh.stereo = stereo = vi->channels-1 ;
+
+        wfh.n_samples = ov_pcm_total(&oggfile,-1) ;
+
+        BYTESPERSAMPLE=2 ;
+        MAXSAMPLEVALUE = 1 << 15 ;
+
+        current_ogg_or_mp3_pos = 0 ;
+	fprintf(stderr, "Oggfile: FRAMESIZE=%d\n", BYTESPERSAMPLE*wfh.n_channels) ;
+	wfh.successful_open = TRUE ;
+    }
+#endif
+
+#ifdef HAVE_MP3
+    if(audio_type == MP3_TYPE) {
+	long rate ;
+	int channels ;
+	int encoding ;
+	mpg123_getformat(fp_mp3,&rate,&channels,&encoding) ;
+	mpg123_scan(fp_mp3) ;
+        wfh.n_samples = mpg123_length(fp_mp3) ;
+        wfh.rate = rate ;
+        wfh.n_channels = channels ;
+        wfh.stereo = stereo = channels-1 ;
+
+
+        BYTESPERSAMPLE=2 ;
+        MAXSAMPLEVALUE = 1 << 15 ;
+
+	off_t pos = mpg123_tell(fp_mp3) ;
+	off_t curr_frame = mpg123_tellframe(fp_mp3) ;
+
+        current_ogg_or_mp3_pos = 0 ;
+	fprintf(stderr, "Mp3file: FRAMESIZE=%d, pos=%d, frame=%d\n", BYTESPERSAMPLE*wfh.n_channels, (int)pos, (int)curr_frame) ;
+	wfh.successful_open = TRUE ;
+    }
+#endif
+
+    FRAMESIZE = BYTESPERSAMPLE*wfh.n_channels ;
+    PLAYBACK_FRAMESIZE = 2*wfh.n_channels ;
+
+    wfh.playback_bits = audio_bits = wfh.bits = BYTESPERSAMPLE*8 ;
+
+    wfh.max_allowed = MAXSAMPLEVALUE-1 ;
+
     gwc_window_set_title(filename) ;
 
     return wfh ;
@@ -547,19 +769,104 @@ struct sound_prefs open_wavefile(char *filename, struct view *v)
 
 void position_wavefile_pointer(long sample_number)
 {
-    sf_seek(sndfile, sample_number, SEEK_SET) ;
+    if(audio_type == SNDFILE_TYPE) {
+	sf_seek(sndfile, sample_number, SEEK_SET) ;
+    } else if(audio_type == MP3_TYPE) {
+#ifdef HAVE_MP3
+        if(current_ogg_or_mp3_pos != sample_number) {
+	    off_t new_pos ;
+            current_ogg_or_mp3_pos = sample_number ;
+	    if(sample_number != 0) {
+		unsigned char buf[1152*4] ;
+		new_pos = mpg123_seek(fp_mp3,sample_number,SEEK_SET) ;
+		off_t curr_frame = mpg123_tellframe(fp_mp3) ;
+/*  		if(curr_frame > 0) curr_frame-- ;  */
+		mpg123_seek_frame(fp_mp3,curr_frame,SEEK_SET) ;
+		int presample_number = (int)mpg123_tell(fp_mp3) ;
+/*  		if(presample_number > 0) presample_number-- ;  */
+		int samples_to_read = sample_number - presample_number ;
+		new_pos = mpg123_seek(fp_mp3,presample_number,SEEK_SET) ;
+
+/*  		    fprintf(stderr, "position_wf_ptr, samples_to_read:%d > 1152!!\n", samples_to_read) ;  */
+/*  		    fprintf(stderr, "curr_frame:%d presample_number:%d\n", curr_frame,presample_number) ;  */
+/*  		    fprintf(stderr, "position_wf_ptr, want:%d got%d\n", (int)sample_number, (int)new_pos) ;  */
+		if(samples_to_read > 1152) {
+		    exit(1) ;
+		}
+
+		unsigned int done ;
+		int err ;
+
+		err = mpg123_read(fp_mp3, buf, samples_to_read*FRAMESIZE, &done) ;
+	    } else {
+		new_pos = mpg123_seek(fp_mp3,sample_number,SEEK_SET) ;
+		nonzero_seek = 0 ;
+	    }
+/*  	    fprintf(stderr, "position_wf_ptr, want:%d got%d\n", (int)sample_number, (int)new_pos) ;  */
+        }
+#endif
+
+    } else {
+#ifdef HAVE_OGG
+        if(current_ogg_or_mp3_pos != sample_number) {
+            fprintf(stderr, "pos_wv_ptr, was %ld, want %ld\n", current_ogg_or_mp3_pos, sample_number) ;
+            ov_pcm_seek(&oggfile, sample_number) ;
+            current_ogg_or_mp3_pos = sample_number ;
+        }
+#endif
+    }
 }
 
 int read_raw_wavefile_data(char buf[], long first, long last)
 {
     long n = last - first + 1 ;
-    int n_read ;
+    int n_read = 0 ;
+    int n_bytes_read = 0 ;
+    int bufsize = n * FRAMESIZE ;
 
     position_wavefile_pointer(first) ;
 
-    n_read = sf_read_raw(sndfile, buf, n*FRAMESIZE) ;
+    if(audio_type == SNDFILE_TYPE) {
+	n_bytes_read = sf_read_raw(sndfile, buf, n*FRAMESIZE) ;
+	return n_bytes_read/FRAMESIZE ;
+    }
 
-    return n_read/FRAMESIZE ;
+#ifdef HAVE_OGG
+    if(audio_type == OGG_TYPE) {
+	int ret ;
+	while(n_read < n) {
+            ret = ov_read(&oggfile, (char *)&buf[n_bytes_read], bufsize-n_bytes_read,0,2,1,&current_ogg_bitstream) ;
+	    if(ret > 0) {
+		n_read += ret/FRAMESIZE ;
+		n_bytes_read += ret ;
+	    } else {
+		break ;
+	    }
+	}
+	current_ogg_or_mp3_pos += n_read ;
+	return n_read ;
+    }
+#endif
+
+#ifdef HAVE_MP3
+    if(audio_type == MP3_TYPE) {
+	size_t done ;
+	int err ;
+	struct mpg123_frameinfo mi ;
+	while(n_read < n) {
+	    err = mpg123_read(fp_mp3, (unsigned char *)&buf[n_bytes_read], bufsize-n_bytes_read, &done) ;
+	    if(err != MPG123_OK) fprintf(stderr, "read had a problem, %d\n", err) ;
+	    err = mpg123_info(fp_mp3, &mi) ;
+/*  	    fprintf(stderr, "fs %d\n", (int)mi.framesize) ;  */
+	    n_bytes_read += done ;
+	    n_read += done/FRAMESIZE ;
+	}
+	current_ogg_or_mp3_pos += n_read ;
+	return n_read ;
+    }
+#endif
+
+    return n_read ;
 }
 
 int write_raw_wavefile_data(char buf[], long first, long last)
@@ -587,8 +894,35 @@ int read_wavefile_data(long left[], long right[], long first, long last)
     bufsize_long = sizeof(audio_buffer) / sizeof(long) ;
 
     while(s_i < n) {
+	long n_read ;
+
+#define TRY_NEW_ABSTRACTION_NOT
+#ifdef TRY_NEW_ABSTRACTION
+	n_read = read_raw_wavefile_data((char *)p_int, first, last) ;
+#else
 	long n_this = MIN((n-s_i)*(stereo+1), bufsize_long) ;
-	long n_read = sf_read_int(sndfile, p_int, n_this) ;
+	if(audio_type == SNDFILE_TYPE) {
+	    n_read = sf_read_int(sndfile, p_int, n_this) ;
+	}
+
+#ifdef HAVE_OGG
+	if(audio_type == OGG_TYPE) {
+            n_read = ov_read(&oggfile, (char *)p_int, n_this*FRAMESIZE,0,2,1,&current_ogg_bitstream) ;
+	    n_read /= FRAMESIZE ;
+	}
+#endif
+
+#ifdef HAVE_MP3
+	if(audio_type == MP3_TYPE) {
+/*  	    size_t done ;  */
+/*  	    int err = mpg123_read(fp_mp3, (unsigned char *)&buf[n_bytes_read], bufsize-n_bytes_read, &done) ;  */
+/*  	    if(err != MPG123_OK) fprintf(stderr, "read had a problem, %d\n", err) ;  */
+/*  	    n_read = done/FRAMESIZE ;  */
+	}
+#endif
+
+#endif
+
 
 	for(j = 0  ; j < n_read ; ) {
 
@@ -607,7 +941,7 @@ int read_wavefile_data(long left[], long right[], long first, long last)
 
 	if(n_read == 0) {
 	    char tmp[100] ;
-	    sprintf(tmp, "Attempted to read past end of audio, first=%ld, last=%ld", first, last) ;
+	    snprintf(tmp, sizeof(tmp), "Attempted to read past end of audio, first=%ld, last=%ld", first, last) ;
 	    warning(tmp) ;
 	    exit(1) ;
 	}
@@ -628,9 +962,88 @@ int read_fft_real_wavefile_data(fftw_real left[], fftw_real right[], long first,
 
     position_wavefile_pointer(first) ;
 
+    if(audio_type != SNDFILE_TYPE) {
+	long pos = first ;
+	short *buffer2 ;
+	int bufsize_short ;
+
+	buffer2 = (short *)audio_buffer2 ;
+	bufsize_short = sizeof(audio_buffer2) / sizeof(short) ;
+
+	while(s_i < n) {
+	    long n_this = MIN((n-s_i), bufsize_short) ;
+	    int n_read = read_raw_wavefile_data((char *)audio_buffer2, pos, pos+n_this-1) ;
+
+	    pos += n_read ;
+
+	    for(j = 0  ; j < n_read ; j++) {
+		left[s_i] = buffer2[j] ;
+		if(stereo) {
+		    j++ ;
+		    right[s_i] = buffer2[j] ;
+		} else {
+		    right[s_i] = left[s_i] ;
+		}
+		left[s_i] /= MAXSAMPLEVALUE ;
+		right[s_i] /= MAXSAMPLEVALUE ;
+		if(first == 0) {
+/*  		    fprintf(stderr, "%4d %d %d\n", (int)s_i, (int)left[s_i], (int)right[s_i]) ;  */
+		}
+		s_i++ ;
+	    }
+
+	    if(n_read == 0) {
+		char tmp[100] ;
+		snprintf(tmp, sizeof(tmp), "read_fft_real Attempted to read past end of audio, first=%ld, last=%ld", first, last) ;
+		warning(tmp) ;
+		//exit(1) ;
+	    }
+
+	}
+    } else {
+	while(s_i < n) {
+	    long n_this = MIN((n-s_i)*(stereo+1), bufsize_double) ;
+	    long n_read = sf_read_double(sndfile, buffer, n_this) ;
+
+	    for(j = 0  ; j < n_read ; j++) {
+		left[s_i] = buffer[j] ;
+		if(stereo) {
+		    j++ ;
+		    right[s_i] = buffer[j] ;
+		} else {
+		    right[s_i] = left[s_i] ;
+		}
+		s_i++ ;
+	    }
+
+	    if(n_read == 0) {
+		char tmp[100] ;
+		snprintf(tmp, sizeof(tmp), "Attempted to read past end of audio, first=%ld, last=%ld", first, last) ;
+		warning(tmp) ;
+		exit(1) ;
+	    }
+
+	}
+    }
+
+    return s_i ;
+}
+
+int read_float_wavefile_data(float left[], float right[], long first, long last)
+{
+    long n = last - first + 1 ;
+    long s_i = 0 ;
+    long j ;
+
+    float *buffer = (float *)audio_buffer ;
+
+    int bufsize_float = sizeof(audio_buffer) / sizeof(float) ;
+
+    position_wavefile_pointer(first) ;
+
     while(s_i < n) {
-	long n_this = MIN((n-s_i)*(stereo+1), bufsize_double) ;
-	long n_read = sf_read_double(sndfile, buffer, n_this) ;
+	long n_this = MIN((n-s_i)*(stereo+1), bufsize_float) ;
+	long n_read = sf_read_float(sndfile, buffer, n_this) ;
 
 	for(j = 0  ; j < n_read ; j++) {
 	    left[s_i] = buffer[j] ;
@@ -645,7 +1058,7 @@ int read_fft_real_wavefile_data(fftw_real left[], fftw_real right[], long first,
 
 	if(n_read == 0) {
 	    char tmp[100] ;
-	    sprintf(tmp, "Attempted to read past end of audio, first=%ld, last=%ld", first, last) ;
+	    snprintf(tmp, sizeof(tmp), "Attempted to read past end of audio, first=%ld, last=%ld", first, last) ;
 	    warning(tmp) ;
 	    exit(1) ;
 	}
@@ -729,6 +1142,43 @@ int write_fft_real_wavefile_data(fftw_real left[], fftw_real right[], long first
 #undef MAXBUF
 }
 
+int write_float_wavefile_data(float left[], float right[], long first, long last)
+{
+    long n = last - first + 1 ;
+    long j ;
+    long n_written = 0 ;
+
+/* make SURE MAXBUF is a multiple of 2 */
+#define MAXBUF 500
+    float buf[MAXBUF] ;
+
+    n_in_buf = 0 ;
+
+    position_wavefile_pointer(first) ;
+
+    for(j = 0  ; j < n ; j++) {
+	buf[n_in_buf] = left[j] ;
+	n_in_buf++ ;
+	if(stereo) {
+	    buf[n_in_buf] = right[j] ;
+	    n_in_buf++ ;
+	}
+
+	if(n_in_buf == MAXBUF) {
+	    n_written += sf_write_float(sndfile, buf, n_in_buf) ;
+	    n_in_buf = 0 ;
+	}
+    }
+
+    if(n_in_buf > 0) {
+	n_written += sf_write_float(sndfile, buf, n_in_buf) ;
+	n_in_buf = 0 ;
+    }
+
+    return n_written/2 ;
+#undef MAXBUF
+}
+
 int write_wavefile_data(long left[], long right[], long first, long last)
 {
     long n = last - first + 1 ;
@@ -820,19 +1270,26 @@ int process_audio(gfloat *pL, gfloat *pR)
     /* for now force playback to 16 bit... */
 #define BYTESPERSAMPLE 2
 
-    if(BYTESPERSAMPLE < 3) {
-	maxpossible = 1 << 15 ;
-	n_read = sf_readf_short(sndfile, p_short, n_samples_to_read) ;
+    if(audio_type == SNDFILE_TYPE) {
+	if(BYTESPERSAMPLE < 3) {
+	    maxpossible = 1 << 15 ;
+	    n_read = sf_readf_short(sndfile, p_short, n_samples_to_read) ;
+	} else {
+	    maxpossible = 1 << 23 ;
+	    n_read = sf_readf_int(sndfile, p_int, n_samples_to_read) ;
+	}
     } else {
-	maxpossible = 1 << 23 ;
-	n_read = sf_readf_int(sndfile, p_int, n_samples_to_read) ;
+#if defined(HAVE_MP3) || defined(HAVE_OGG)
+	maxpossible = 1 << 15 ;
+	n_read = read_raw_wavefile_data((char *)p_char, current_ogg_or_mp3_pos, current_ogg_or_mp3_pos+n_samples_to_read-1) ;
+#endif
     }
 
 #define FEATHER_WIDTH 30000
     if(playback_samples_remaining - n_read < 1) {
 	feather_out = 1 ;
 	feather_out_N = MIN(n_read, FEATHER_WIDTH) ;
-	printf("Feather out n_read=%d, playback_samples_remaining=%d, N=%lf\n", n_read, playback_samples_remaining, feather_out_N) ;
+	fprintf(stderr, "Feather out n_read=%ld, playback_samples_remaining=%ld, N=%lf\n", n_read, playback_samples_remaining, feather_out_N) ;
     }
 
     for(frame = 0  ; frame < n_read ; frame++) {
@@ -855,7 +1312,7 @@ int process_audio(gfloat *pL, gfloat *pR)
 		//    printf("%hd %hd\n", p_short[i], p_short[i+1]) ;
 		//}
 
-		if(frame == n_read-1) printf("Feather out final %lf, n_read=%d", p, n_read) ;
+		if(frame == n_read-1) fprintf(stderr, "Feather out final %lf, n_read=%ld", p, n_read) ;
 	    }
 
 	    vl = p_short[i] ;
@@ -901,13 +1358,14 @@ int process_audio(gfloat *pL, gfloat *pR)
 	    extern int audio_is_looping ;
 
 	    if(audio_is_looping == FALSE) {
-		char zeros[1024] ;
+		unsigned char zeros[1024] ;
 		long zeros_needed ;
 		memset(zeros,0,sizeof(zeros)) ;
 		audio_state = AUDIO_IS_PLAYBACK ;
 		audio_playback = FALSE ;
 
 		zeros_needed = playback_bytes_per_block - (playback_total_bytes % playback_bytes_per_block) ;
+		if(zeros_needed < PLAYBACK_FRAMESIZE) zeros_needed = PLAYBACK_FRAMESIZE ; 
 		do {
 		    len = audio_device_write(zeros, MIN(zeros_needed, sizeof(zeros))) ;
 		    zeros_needed -= len ;
@@ -956,5 +1414,5 @@ void stop_playback(int force)
 /*      fprintf(stderr, "Done usleeping 300000\n") ;  */
 
     audio_state = AUDIO_IS_IDLE ;
-    audio_device_close() ;
+    audio_device_close(1-force) ;
 }
