@@ -79,6 +79,9 @@ int only_blank(char *str)
    }
    return 1;
 }
+
+static int blanks_marked ; /* song markers are positioned at start AND end of each song? */
+
 #define ARRAYSIZE(x) (sizeof(x) / sizeof(x[0]))
 /* CD Tracks must be multiple of 588 or they will be padded with zeros */
 #define SONG_BLOCK_LEN 588
@@ -160,7 +163,7 @@ void cdrdao_toc_info(char *filename)
     gtk_widget_show (song_table);
 
     row = 0;
-    for (j = 0; j < num_song_markers+1; j++) {
+    for (j = 0; j < num_song_markers+1; j += 1+blanks_marked) {
        for (i = 0; i < ARRAYSIZE(song_info); i++) {
           char *init;
 
@@ -187,7 +190,6 @@ void cdrdao_toc_info(char *filename)
 	   warning(buf);
 	} else {
 	   int found_text = 0;
-	   long start = 0;
 	   for (i = 0; i < ARRAYSIZE(album_info); i++) {
 	       char *str;
 	       str = (char *)gtk_entry_get_text(GTK_ENTRY(album_info[i].widget));
@@ -196,7 +198,7 @@ void cdrdao_toc_info(char *filename)
 	       if (strlen(album_info[i].init) == 0 && !only_blank(str))
 		  found_text = 1;;
 	   }
-	   for (j = 0; j < num_song_markers+1; j++) {
+	   for (j = 0; j < num_song_markers+(1-blanks_marked); j += 1+blanks_marked) {
 	      for (i = 0; i < ARRAYSIZE(song_info); i++) {
 		  char *str;
 		  str = (char *)gtk_entry_get_text(GTK_ENTRY(song_widget[j][i]));
@@ -230,13 +232,23 @@ void cdrdao_toc_info(char *filename)
 	      new_cdtext = NULL;
 	   }
 
-	   for (j = 0; j < num_song_markers+1; j++) {
-	      int end;
-	      if (j == num_song_markers) {
-		 end = prefs.n_samples-1;
-              } else {
-                 end = ((song_markers[j] + 2324 / 2) / 2324) * 2324;
-              }
+	   for (j = 0; j < num_song_markers+(1-blanks_marked); j += 1+blanks_marked) {
+	      long end;
+	      long start = song_markers[j] ;
+	      long length ;
+	      int j1 = j+1 ;
+
+	      if(j1 < num_song_markers) {
+		  length = song_markers[j+1] - start + 1 ;
+	      } else {
+		  length = (prefs.n_samples-1) - start + 1 ;
+	      }
+
+#define AUDIO_BLOCK_LEN 588
+	      length = ((length + AUDIO_BLOCK_LEN / 2) / AUDIO_BLOCK_LEN) * AUDIO_BLOCK_LEN ;
+
+	      end = start + length -1 ;
+
 	      fprintf(toc, "TRACK AUDIO\n");
 	      if (found_text) {
 		 fprintf(toc, "   CD_TEXT {\n      LANGUAGE 0 {\n");
@@ -256,7 +268,6 @@ void cdrdao_toc_info(char *filename)
 		 fprintf(toc, "      }\n   }\n");
 	      }
 	      fprintf(toc, "   FILE \"%s\" %ld %ld\n", wave_filename, start, end - start) ;
-	      start = end;
 
 	   }
 	   fclose(toc);
@@ -305,8 +316,6 @@ void store_cdrdao_toc(GtkFileSelection * selector, gpointer user_data)
 
 }
 
-
-
 void save_cdrdao_toc(GtkWidget * widget, gpointer data)
 {
    char pathname[256] = "./cdrdao.toc";
@@ -342,13 +351,25 @@ void save_cdrdao_toc(GtkWidget * widget, gpointer data)
    }
 }
 
-void add_song_marker(void)
+void save_cdrdao_tocs(GtkWidget * widget, gpointer data)
+{
+    blanks_marked = 0 ;
+    save_cdrdao_toc(widget, data) ;
+}
+
+void save_cdrdao_tocp(GtkWidget * widget, gpointer data)
+{
+    blanks_marked = 1 ;
+    save_cdrdao_toc(widget, data) ;
+}
+
+int _add_song_marker(long loc)
 {
    int i,j;
-   long loc = audio_view.selected_first_sample;
 
    if (num_song_markers >= MAX_MARKERS - 1) {
       set_status_text("No more song markers available");
+      return 0 ;
    } else {
       for (i = 0; i < num_song_markers; i++) {
          if (song_markers[i] > loc) {
@@ -358,15 +379,33 @@ void add_song_marker(void)
       for (j = num_song_markers - 1; j >= i;  j--) {
          song_markers[j+1] = song_markers[j];
       }
-      /* commenting out -- cdrdao will automatically pad the audio data
-         so this seems an unnecessary constraint.  jw 1/5/2005 */
-      //song_markers[i] = (loc / SONG_BLOCK_LEN) * SONG_BLOCK_LEN;
       song_markers[i] = loc ;
       num_song_markers++; 
-      main_redraw(FALSE, TRUE);
+      return 1 ;
    }
 }
 
+void add_song_marker(void)
+{
+   long loc = audio_view.selected_first_sample;
+
+   if(_add_song_marker(loc))
+      main_redraw(FALSE, TRUE);
+}
+
+void add_song_marker_pair(void)
+{
+   int r ;
+   long loc = audio_view.selected_first_sample;
+   r = _add_song_marker(loc) ;
+
+   loc = audio_view.selected_last_sample;
+   r += _add_song_marker(loc) ;
+
+   if(r > 0)
+      main_redraw(FALSE, TRUE);
+}
+
 void delete_song_marker(void)
 {
    int i,j;
@@ -450,6 +489,25 @@ void select_song_marker(void)
    }
 }
 
+
+#define DETECT_GAPS_NOT
+#ifdef DETECT_GAPS
+double sample_rms(struct sample_block *sample_buffer, int i, int w)
+{
+    int istart=i-w/2 ;
+    if(istart < 0) istart=0 ;
+    int iend=i+w/2 ;
+    double sum=0 ;
+    double n=iend-istart+1.0 ;
+
+    for(i=istart ; i <= iend ; i++) {
+	sum += (sample_buffer[i].rms[0]+sample_buffer[i].rms[1])/2.0 ;
+    }
+
+    return sum/n ;
+}
+#endif
+
 void mark_songs(GtkWidget * widget, gpointer data)
 {
     struct sample_block *sample_buffer ;
@@ -567,8 +625,10 @@ void mark_songs(GtkWidget * widget, gpointer data)
     silence_cntr = 0;
     last_silence = 0;
     last_song_block = 0;
+
     for (i = 0; i < avg_blocks; i++)
 	delay[i] = 0.0;
+
     for (i = min_song_blocks; i < n_blocks - min_song_blocks; i++) {
        song_amp = (sample_buffer[i].max_value[0] + sample_buffer[i].max_value[1]);
        song_window_amp += song_amp - delay[delay_cntr];
@@ -597,6 +657,47 @@ void mark_songs(GtkWidget * widget, gpointer data)
 	  }
        }
     }
+
+
+#ifdef DETECT_GAPS
+    if(1) {
+	/* this attempts to detect entire gaps between songs, it does not work
+	 * well yet and should be disabled except for testing... */
+	long tmp_markers[MAX_MARKERS] ;
+	int n_tmp = num_song_markers ;
+
+	for(i = 0 ; i < num_song_markers ; i++)
+	    tmp_markers[i] = song_markers[i] ;
+
+	num_song_markers=0 ;
+
+	for(i=0 ; i < n_tmp ; i++) {
+	    int iblk = tmp_markers[i]/SBW ;
+#define bsw 256
+#define hbsw 64
+	    double midpt_rms = sample_rms(sample_buffer,iblk,bsw) ;
+	    int j ;
+
+	    for(j = hbsw ; j < min_song_blocks ; j += hbsw) { 
+		double lead_rms = sample_rms(sample_buffer,iblk-j,bsw) ;
+		if(lead_rms > midpt_rms*1.3)
+		    break ;
+	    }
+
+	    if(j > hbsw)
+		song_markers[num_song_markers++] = (iblk-j)*SBW ;
+
+	    for(j = hbsw ; j < min_song_blocks ; j += hbsw) { 
+		double tail_rms = sample_rms(sample_buffer,iblk+j,bsw) ;
+		if(tail_rms > midpt_rms*1.5)
+		    break ;
+	    }
+
+	    if(j > hbsw)
+		song_markers[num_song_markers++] = (iblk+j)*SBW ;
+	}
+    }
+#endif
    
     snprintf(buf, sizeof(buf), "Marked %ld songs", num_song_markers + 1);
     set_status_text(buf);
