@@ -683,7 +683,7 @@ void set_misc_preferences(GtkWidget * widget, gpointer data)
 
     audio_device_entry =
 	add_number_entry_with_label(audio_device,
-			   "Audio device", dialog_table, row++);
+			   "Audio device (try /dev/dsp for OSS, hw:0,0 or hw 1,0 ... for ALSA", dialog_table, row++);
 
 
     gtk_box_pack_start(GTK_BOX(GTK_DIALOG(dlg)->vbox), dialog_table,
@@ -1020,7 +1020,7 @@ void load_denoise_preferences(void)
     denoise_prefs.min_sample_freq = gnome_config_get_float("min_sample_freq=0.0");
     denoise_prefs.max_sample_freq = gnome_config_get_float("max_sample_freq=44100.0");
     denoise_prefs.noise_suppression_method =
-	gnome_config_get_int("noise_suppression_method=1");
+    gnome_config_get_int("noise_suppression_method=1");
     gnome_config_pop_prefix();
 }
 
@@ -1056,8 +1056,15 @@ void fft_window_select(GtkWidget * clist, gint row, gint column,
 	window_type = DENOISE_WINDOW_BLACKMAN_HYBRID;
     if (row == 2)
 	window_type = DENOISE_WINDOW_HANNING_OVERLAP_ADD;
+#ifdef DENOISE_TRY_ONE_SAMPLE
+    if (row == 3)
+	window_type = DENOISE_WINDOW_ONE_SAMPLE;
+    if (row == 4)
+	window_type = DENOISE_WINDOW_WELTY;
+#else
     if (row == 3)
 	window_type = DENOISE_WINDOW_WELTY;
+#endif
 }
 
 void noise_method_window_select(GtkWidget * clist, gint row, gint column,
@@ -1073,6 +1080,8 @@ void noise_method_window_select(GtkWidget * clist, gint row, gint column,
 	noise_suppression_method = DENOISE_LORBER;
     if (row == 4)
 	noise_suppression_method = DENOISE_WOLFE_GODSILL;
+    if (row == 5)
+	noise_suppression_method = DENOISE_EXPERIMENTAL ;
 }
 
 void denoise_set_preferences(GtkWidget * widget, gpointer data)
@@ -1094,20 +1103,28 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
     GtkWidget *fft_window_list;
 
     gchar *fft_window_titles[] = { "Windowing Function" };
+#ifdef DENOISE_TRY_ONE_SAMPLE
+    gchar *fft_window_parms[4][1] = { {"Blackman"},
+    {"Hybrid Blackman-Full Pass"},
+    {"Hanning-overlap-add (Best)"},
+    {"Hanning-one-sample-shift (Experimental)"}
+#else
     gchar *fft_window_parms[3][1] = { {"Blackman"},
     {"Hybrid Blackman-Full Pass"},
     {"Hanning-overlap-add (Best)"}
+#endif
     };
 
     GtkWidget *noise_method_window_list;
 
     gchar *noise_method_window_titles[] = { "Noise Suppresion Method" };
-    gchar *noise_method_window_parms[5][1] = {
+    gchar *noise_method_window_parms[6][1] = {
 						{"Weiner"},
 						{"Power Spectral Subtraction"},
 						{"Ephraim-Malah 1984"},
 						{"Lorber & Hoeldrich (Best)"},
-						{"Wolfe & Godsill (Experimental)"}
+						{"Wolfe & Godsill (Experimental)"},
+						{"Extremely Experimental"}
 					    };
 
     load_denoise_preferences();
@@ -1119,6 +1136,9 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
     gtk_clist_append(GTK_CLIST(fft_window_list), fft_window_parms[0]);
     gtk_clist_append(GTK_CLIST(fft_window_list), fft_window_parms[1]);
     gtk_clist_append(GTK_CLIST(fft_window_list), fft_window_parms[2]);
+#ifdef DENOISE_TRY_ONE_SAMPLE
+    gtk_clist_append(GTK_CLIST(fft_window_list), fft_window_parms[3]);
+#endif
     gtk_clist_select_row(GTK_CLIST(fft_window_list),
 			 denoise_prefs.window_type, 0);
     gtk_signal_connect(GTK_OBJECT(fft_window_list), "select_row",
@@ -1140,6 +1160,8 @@ void denoise_set_preferences(GtkWidget * widget, gpointer data)
 		     noise_method_window_parms[3]);
     gtk_clist_append(GTK_CLIST(noise_method_window_list),
 		     noise_method_window_parms[4]);
+    gtk_clist_append(GTK_CLIST(noise_method_window_list),
+		     noise_method_window_parms[5]);
     gtk_clist_select_row(GTK_CLIST(noise_method_window_list),
 			 denoise_prefs.noise_suppression_method, 0);
     gtk_signal_connect(GTK_OBJECT(noise_method_window_list), "select_row",
