@@ -41,6 +41,11 @@ static char current_undo_msg[200] ;
 static int undo_fd = -1 ;
 static int undo_level = 0 ;
 
+int get_undo_levels(void)
+{
+    return undo_level ;
+}
+
 int start_save_undo(char *undo_msg, struct view *v)
 {
     char filename[1024] ;
@@ -55,7 +60,7 @@ int start_save_undo(char *undo_msg, struct view *v)
 	return -1 ;
     }
 
-    if(undo_level == 0)
+    if(undo_level == 1)
 	strcpy(current_undo_msg, "Nothing to undo") ;
 
     l = strlen(current_undo_msg) ;
@@ -97,7 +102,7 @@ int save_undo_data(long first_sample, long last_sample, struct sound_prefs *p, i
 	char buf[200] ;
 	int ret ;
 
-	sprintf(buf, "Undo will need %7.2f Mbytes of disk space", n_sample*FRAMESIZE/1000000.0) ;
+	sprintf(buf, "Undo will need %7.2f Mbytes of disk space (skipping undo commits changes to your original audio file)", n_sample*FRAMESIZE/1000000.0) ;
 
 	dialog = gtk_dialog_new_with_buttons(buf, NULL, GTK_DIALOG_DESTROY_WITH_PARENT,
 	   "Skip undo", 0, "Cancel edit action", 1, "Save undo data", 2, NULL) ;
@@ -197,7 +202,7 @@ int undo(struct view *v, struct sound_prefs *p)
     const int BLOCK_SIZE = 1024 ;
     char buf[BLOCK_SIZE * FRAMESIZE] ;
     long blocks ;
-    long *data_start_pos ;
+    off_t *data_start_pos ;
     long total_sections;
 #ifndef TRUNCATE_OLD
     int undo_type;
@@ -228,8 +233,8 @@ int undo(struct view *v, struct sound_prefs *p)
     v->selection_region = TRUE ;
 
     n_sections = 0 ;
-    data_start_pos = (long *)calloc(n_sections_max, sizeof(long)) ;
-    data_start_pos[n_sections] = lseek(undo_fd, 0, SEEK_CUR) ;
+    data_start_pos = (off_t *)calloc(n_sections_max, sizeof(off_t)) ;
+    data_start_pos[n_sections] = lseek(undo_fd, (off_t)0, SEEK_CUR) ;
 
     while(read(undo_fd, (char *)&first_sample, sizeof(first_sample)) == sizeof(first_sample) ) {
 	read(undo_fd, (char *)&last_sample, sizeof(last_sample)) ;
@@ -244,13 +249,13 @@ int undo(struct view *v, struct sound_prefs *p)
 #ifndef TRUNCATE_OLD
         if (undo_type != UNDO_INSERT)
 #endif
-        lseek(undo_fd, (last_sample - first_sample + 1) * FRAMESIZE, SEEK_CUR);
+        lseek(undo_fd, (off_t)((last_sample - first_sample + 1) * FRAMESIZE), SEEK_CUR);
 	n_sections++ ;
 	if(n_sections == n_sections_max) {
 	    n_sections_max += N_ALLOC_INC ;
-	    data_start_pos = (long *)realloc(data_start_pos, n_sections_max*sizeof(long)) ;
+	    data_start_pos = (off_t *)realloc(data_start_pos, n_sections_max*sizeof(off_t)) ;
 	}
-	data_start_pos[n_sections] = lseek(undo_fd, 0, SEEK_CUR) ;
+	data_start_pos[n_sections] = lseek(undo_fd, (off_t)0, SEEK_CUR) ;
     }
 
 
